/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Js component                                                         **
**                                                                       **
**	Copyright(C) 2018 by George Malamas                                  **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: June 29, 2018                                             **
**                                                                       **
**	Release number: 2                                                    **
**                                                                       **
***************************************************************************
*/
// This our js component, in which is implemented our main observer, which is always registered if there is
// at least one browser window.
// Our observer is used in appVer >= 3 and has three purposes:
// 1. In appVer 3 - 19, gets the restart request notification and help us to patch "on the fly" the
//    embedded restart dialog of browser. Details for this can be found in our main code file.
// 2. In appVer >= 20, gets any normal restart request notification, cancels it and helps us take the
//    control, letting the user decide what to do, in accordance the restart options we offer.
// 3. In appVer >= 20, gets any normal exit request notification and depending on conditions, does the following:
//    In private browsing mode, cancels the request and passes the control to us.
//    In non-private browsing mode, helps us to perform any pending actions we do in some cases on app exiting
//    or restarting. 
//    All the details about, can be found in our main code file.
// Our js component also, is used as a "Per session" global data storage.

"use strict";

var appVersion = parseFloat(Components.classes["@mozilla.org/xre/app-info;1"]
				.getService(Components.interfaces.nsIXULAppInfo).version);

// Factory object
var RestartRequestObserverServiceFactory = {
	instance: null,
	createInstance: function RFSF_createInstance(outer, iid) {
		if(outer != null) {
			throw Components.results.NS_ERROR_NO_AGGREGATION;
		}
			return this.instance == null ?
			this.instance = new RestartRequestObserver() : this.instance;
	}
};

// Class constructor
function RestartRequestObserver() {
	this.wrappedJSObject = this;
	this.observerRegister();
	if(appVersion>=20) {
		this.getPrvBrowsingAutostart();
	};
};

// Class definition
RestartRequestObserver.prototype = {
	classDescription: "Restart manager - Restart Request Observer",
	contractID: "@restart_manager/chania.gr;1",
	classID: Components.ID("{d6ecd398-dad8-4f6f-9edb-95822afaf179}"),
	_xpcom_factory: RestartRequestObserverServiceFactory,
	_xpcom_categories: [{
						category: "profile-after-change", service: true}, // Gecko >= 2.0
						{category: "app-startup", service: true // Gecko = 1.9.x
						}], //

	obsSrvc:      Components.classes["@mozilla.org/observer-service;1"]
					.getService(Components.interfaces.nsIObserverService),
	prefSrvc:     Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService),
	promptSrvc:   Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
					.getService(Components.interfaces.nsIPromptService),
	rmPrefBranch: Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("extensions.restartmanager."),
	brPrefBranch: Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("browser."),
	winMed:       Components.classes["@mozilla.org/appshell/window-mediator;1"]
					.getService(Components.interfaces.nsIWindowMediator),

// Global variables, data storage
	observerIsRegistered: false, // Restart request observer.
	restartFlag: false, // Used as identifier of restart dialog loading in appVer 3-19.
	allowCheck_prvBrowsingAutostart: true, // appVer >= 20.

	getSetVal: function(prop, gsv) {
		if(gsv == "getV") { // get value.
			return this[prop];
		};
		if(gsv == "setT") { // set true.
			this[prop] = true;
			return;
		};
		if(gsv == "setF") { // set false.
			this[prop] = false;
			return;
		};
	},

// Component implementation
  	observe: function RestMan_RestartRequestObserver(aSubject, aTopic, aData) {
		var codeMonitoring = this.rmPrefBranch.getBoolPref("alertsEnabled");
		if(codeMonitoring) {
			this.promptSrvc.alert(null, "Observe:", "Subject: " + aSubject + "\nTopic: " +
				aTopic + "\nData: " + aData);
		};
		var currentBrWin = this.winMed.getMostRecentWindow("navigator:browser");
		if((aSubject instanceof Components.interfaces.nsISupportsPRBool) && aSubject.data) {
			return false; // If the quit request has been canceled by someone (..?)
 		} else if(aTopic == "quit-application-requested" && aData == "restart") {
			if(codeMonitoring) {
				this.promptSrvc.alert(null, "Observe on restart request:", "Subject: " +
					aSubject + "\nTopic: " + aTopic + "\nData: " + aData);
			};
			try {
				if(appVersion>=20) {
					if(this.winMed.getMostRecentWindow("Browser:Preferences") ||
					   (appVersion>=27 && currentBrWin.content.document.documentURI.startsWith("about:support"))) {
					   this.observerUnregister();
					   return;
					} else {
						aSubject.QueryInterface(Components.interfaces.nsISupportsPRBool);
						aSubject.data = true;
						currentBrWin._RestMan._RestManOnRestartRequest("restart", "", "rro");
					};
				} else if(appVersion<20 && appVersion>3) {
					if(this.brPrefBranch.getBoolPref("warnOnRestart") && !Components.classes["@mozilla.org/privatebrowsing;1"]
						.getService(Components.interfaces.nsIPrivateBrowsingService).privateBrowsingEnabled) { // No dialog in private browsing mode.
						this.restartFlag = true;
					};
				} else if(appVersion<3.5) { // AppVer 3
					if(this.brPrefBranch.getBoolPref("warnOnRestart")) {
						this.restartFlag = true;
					};
				};
			} catch(ex) {
				Components.utils.reportError("RestMan - Error report: " + ex);
			};
		} else if(appVersion>=20) {
			if(aTopic == "quit-application-requested" && aData == null) { // On "File --> Exit"
				if(codeMonitoring) {
					this.promptSrvc.alert(null, "Observe on quit request:", "Subject: " +
						aSubject + "\nTopic: " + aTopic + "\nData: " + aData);
				};
				try {
					if(currentBrWin._RestMan.detectOpenDlgs) {
						if(currentBrWin._RestMan.dialogDetector()) {
							aSubject.QueryInterface(Components.interfaces.nsISupportsPRBool);
							aSubject.data = true;
							currentBrWin._RestMan.openedDialog.focus();
							return;
						};
					};
					currentBrWin._RestMan.winTabCounter("", "rro");
					if(currentBrWin._RestMan.allWinPrivate || currentBrWin._RestMan.allTabsPrivate) {
						aSubject.QueryInterface(Components.interfaces.nsISupportsPRBool);
						aSubject.data = true;
						currentBrWin._RestMan._RestManOnRestartRequest("exit", "handleExit", "rro");
					} else {
						currentBrWin._RestMan.onQuitRequest("", "rro");
					};
				} catch(ex) {
					Components.utils.reportError("RestMan - Error report: " + ex);
				};
			};
/*			It's a better method to do our actions on app exiting, but we are not yet sure if we have the
			"lastwindow" notification on closing of all type of last windows, in private and non-private
			browsing mode, in all supporting app versions..
			if(aTopic == "quit-application-requested" && aData == "lastwindow") { // Last window is closing.
				if(codeMonitoring) {
					this.promptSrvc.alert(null, "Observe on quit request:", "Subject: " +
						aSubject + "\nTopic: " + aTopic + "\nData: " + aData);
				};
				try {
					currentBrWin._RestMan.onQuitRequest();
				} catch(ex) {
					Components.utils.reportError("RestMan - Error report: " + ex);
				};
			}; */
		};
	},

	observerRegister: function() {
		var codeMonitoring = this.rmPrefBranch.getBoolPref("alertsEnabled");
		if(!(this.observerIsRegistered)) {
			this.obsSrvc.addObserver(this, "quit-application-requested", false);
			this.observerIsRegistered = true;
			if(codeMonitoring) {
				this.promptSrvc.alert(null, "Restart Request Observer:", "Registered");
			};
		};
	},

	observerUnregister: function() {
		var codeMonitoring = this.rmPrefBranch.getBoolPref("alertsEnabled");
		if(this.observerIsRegistered) {
			this.obsSrvc.removeObserver(this, "quit-application-requested");
			this.observerIsRegistered = false;
			if(codeMonitoring) {
				this.promptSrvc.alert(null, "Restart Request Observer:", "Unregistered");
			};
		};
    },

	getPrvBrowsingAutostart: function() { // AppVer >= 20.
	// It is used only if "codeMonitoring" is enabled, to determine if the alert message
	// of "sessionSaveHelper" can be shown or not, on app exit/restart/close last window.
	// If app starts with "privatebrowsing.autostart = true", we want this function to return
	// always false, because there are not any savable data (non-private) to be saved on
	// exit or restart.
		if(this.brPrefBranch.getBoolPref("privatebrowsing.autostart")) {
			this.allowCheck_prvBrowsingAutostart = false;
		};
		if(this.allowCheck_prvBrowsingAutostart) {
			if(!this.brPrefBranch.getBoolPref("privatebrowsing.autostart")) {
				return false;
			} else if(this.brPrefBranch.getBoolPref("privatebrowsing.autostart")) {
				return true;
			};
		} else if(!this.allowCheck_prvBrowsingAutostart) {
			return true;
		};
	}
};

// Component registration
if(appVersion>=3) {
	// We have to fool appVer 1.5, which can't see in the future.. Otherwise it produces syntax error.
	Components.utils["import"]("resource://gre/modules/XPCOMUtils.jsm", this);
//	if(appVersion>=20) { // It works, but causes almost quadruple of memory commitment..
//		Components.utils["import"]("chrome://restartmanager/content/restmanJS.js");
//	};
	if(this.XPCOMUtils.generateNSGetFactory) {
		this.NSGetFactory = this.XPCOMUtils.generateNSGetFactory([RestartRequestObserver]); // Gecko >= 2.0
	} else if(this.XPCOMUtils.generateNSGetModule) {
	this.NSGetModule = this.XPCOMUtils.generateNSGetModule([RestartRequestObserver]); // Gecko = 1.9.x
	};
};

// We call our js component this way:
// Components.classes["@restart_manager/chania.gr;1"].getService().wrappedJSObject;

/*******************************************************************************************/
